;--------------------------------------------------------------------------------------------- 
; DECODIFICADOR DE CONTROLE REMOTO INFRAVERMELHO                                    25/02/2016
;
; STATUS: FUNCIONA  -  Familias 16F e 18F
;
; Este programa decodifica os controles remotos listados abaixo e mostra o resultado no LCD.
;
; TV SANYO
; TV TOSHIBA CT-8010
; DVD GRADIENTE CR-203
;
;--------------------------------------------------------------------------------------------- 
; Incluir os registradores e simbolos no programa:
;
;rmtgcount      res      1              ; contador de 1/2 periodos
;#define        rmtgtris TRISB          ; I/O 
;#define        rmtgport PORTB          ; PORTA 
;rmtgbit        equ      0              ; BIT 
;rmtgbuf        res      .4             ; buffer de leitura    
;--------------------------------------------------------------------------------------------- 
; Entrada: Nao ha
;
; Saida  : 'rmtgbuf' contera 32 bits:
;           rmtgbuf+0 --> codigo da tecla no formato complementado
;           rmtgbuf+1 --> codigo da tecla no formato direto
;           rmtgbuf+2 e rmtgbuf+3 --> codigo do device do aparelho
;   
;--------------------------------------------------------------------------------------------- 
; SINAL DO CONTROLE REMOTO
;
; O sinal do controle remoto e' modulado numa portadora de aproximadamente
; 36KHZ que, antes de entrar neste programa, foi retirada pelo circuito
; detetor. Apos a retirada da portadora de 36KHZ, o sinal tem o seguinte
; formato:
;
;                               
;                               [<----------------- 32 bits ------------->]
;
;         HEADER                 DADO  VALOR  DADO    VALOR   DADO   VALOR
;    +--------------+           +-----+     +-----+          +-----+       +-----+
;    |    9,4ms     |           |0,8ms|0,8ms|0,8ms|   1,6ms  |0,8ms|       |0,8ms|  
;    |              |           |     |     |     |          |     |       |     |  
;    |              |           |     |     |     |          |     |       |     |  
;    |              |   BREAK   |     |     |     |          |     |       |     |   STOP
; ---+              +-----------+     +-----+     +----------+     +..   ..+     +------------
;                       2,0ms            0              1
;              
; Ao pressionar uma tecla do controle remoto este emitira o sinal acima onde:
;                 
; HEADER e' um nivel alto de 9,4ms (milisegundos) que indica o inicio da transmissao.
; BREAK  e' um nivel baixo de 2,0ms que indica que vai chegar um bit de dado.
; O nivel alto de 0,8ms apos o BREAK, indica que o nivel baixo de 0,8ms e' um bit de
; valor 0 (zero). Se o nivel baixo tiveer duracao de 1,6ms, entao o bit tera o 
; valor 1. 
; Apos enviar 32 bits de DADOS, sera enviado um nivel baixo de duracao muito 
; grande que significa fim da transmissao (STOP).
; Se a tecla do controle permanecer pressionada, o processo se repetira desde o
; envio do HEADER.
; 
; Para fazer a decodificacao do sinal, contaremos o numero de meios periodos de 
; duracao do sinal de DADO, isto 'e, 0,4ms. Assim, o HEADER de 9,4ms contera
; 23,5 meios periodos; o BREAK contera 5,0 meios periodos; DADO contera 2,0 meios
; periodos e VALOR contera 2,0 meios periodos.
;
;--------------------------------------------------------------------------------------------- 


; --------------------------------------------------------------------------
; Espera chegar o Header (nivel 1) do controle remoto.
; --------------------------------------------------------------------------
rmct_generico  
   if __FAMILIA==16
               bsf     STATUS, RP0
   endif 
               bsf     rmtgtris, rmtgbit         ; rmtgbit como entrada
   if __FAMILIA==16
               bcf     STATUS, RP0
   endif

not_a_head     clrf    rmtgcount                 ; contador de meios periodos
wait_head      movf    rmtgport, w               ; le a porta
               movwf   R0                        ; salva leitura
               btfss   R0, rmtgbit               ; recebeu Header do controle remoto???
               goto    wait_head                 ; NAO --> continua esperando um Header
; --------------------------------------------------------------------------
; Chegou o Header!!!
; Espera chegar um Break (nivel 0) do controle remoto enquanto esta dentro
; de um Header que deve durar no maximo 23 meios periodos.
; --------------------------------------------------------------------------
wait_break     call    wait_halfp                ; espera passar meio periodo
               movf    rmtgport, w               ; le a porta
               movwf   R0                        ; salva leitura
               btfsc   R0, rmtgbit               ; recebeu Break do controle remoto???
               goto    wait_break                ; NAO --> continua esperando um Break
               movlw   .20                       ; w = 20
               subwf   rmtgcount, w              ; w = rmtgcount - 20
               btfss   STATUS,C                  ; w >= 0 ???
               goto    not_a_head                ; NAO --> nao estava dentro de um Header
; --------------------------------------------------------------------------
; Chegou o Break!!!
; Espera chegar um Bit de dado do controle remoto enquanto esta dentro
; de um Break que deve durar no maximo 5 meios periodos.
; --------------------------------------------------------------------------
wait_bit       call    wait_halfp                ; espera passar meio periodo
               movf    rmtgport, w                ; le a porta
               movwf   R0                        ; salva leitura
               btfss   R0, rmtgbit                ; recebeu um bit do controle remoto???
               goto    wait_bit                  ; NAO --> continua esperando um Break
               movlw   .4                        ; w = 4
               subwf   rmtgcount, w                ; w = rmtgcount - 4
               btfss   STATUS,C                  ; w >= 0 ???
               goto    not_a_head                ; NAO --> nao estava dentro de um Break
; --------------------------------------------------------------------------
; Chegou o primeiro bit apos um Break!!!
; Um bit 0 (zero) dura 4 meios periodos sendo 2 em nivel 1 e 2 em nivel 0.
; Um bit 1 (um)   dura 6 meios periodos sendo 2 em nivel 1 e 4 em nivel 0.
; --------------------------------------------------------------------------
wait_bit0      call    wait_halfp                ; espera passar meio periodo
               movf    rmtgport, w                ; le a porta
               movwf   R0                        ; salva leitura
               btfsc   R0, rmtgbit                ; recebeu um bit==0 do controle remoto???
               goto    wait_bit0                 ; NAO --> espera receber 0
               clrf    rmtgcount                   ; limpa contador de meios periodos
wait_bit1      call    wait_halfp                ; espera passar meio periodo
               movf    rmtgport, w                ; le a porta
               movwf   R0                        ; salva leitura
               btfss   R0, rmtgbit                ; recebeu um bit==1 controle remoto???
               goto    wait_bit1                 ; NAO --> espera receber 1
               movlw   .10                       ; w = 10      
               subwf   rmtgcount, w                ; w = rmtgcount - 10
               btfsc   STATUS, C                 ; w < 0 ???
               retlw   0                         ; NAO --> recebeu stop
               clrf    R3                        ; esperamos o valor 0 (zero)
               movlw   .3                        ; w = 3
               subwf   rmtgcount, w                ; w = rmtgcount - 3
               btfsc   STATUS, C                 ; w >= 0 ???
               incf    R3, f                     ; SIM --> veio valor 1

           if __FAMILIA==16
               rrf     R3, f                     ; empurra valor para o Carry 
               rrf     rmtgbuf+.0, f             ; empurra valor para o buffer
               rrf     rmtgbuf+.1, f             ; empurra valor para o buffer
               rrf     rmtgbuf+.2, f             ; empurra valor para o buffer
               rrf     rmtgbuf+.3, f             ; empurra valor para o buffer
           endif

           if __FAMILIA==18
               rrcf     R3, f                     ; empurra valor para o Carry 
               rrcf     rmtgbuf+.0, f             ; empurra valor para o buffer
               rrcf     rmtgbuf+.1, f             ; empurra valor para o buffer
               rrcf     rmtgbuf+.2, f             ; empurra valor para o buffer
               rrcf     rmtgbuf+.3, f             ; empurra valor para o buffer
           endif

               goto    wait_bit0                 ; continua recepcao...


; --------------------------------------------------------------------------
; Espera passar meio periodo (400us) onde cada periodo corresponde a 
; 0,8ms ou 800us
; --------------------------------------------------------------------------
wait_halfp     movlw   .200
               call    delay_us
               movlw   .200
               call    delay_us
               incf    rmtgcount, f
               return

